@@ -10,9 +10,8 @@ module ErrorMessages
 import Prelude.Compat
 
 import Data.Aeson (FromJSON(..), Value, json)
-import Data.Aeson.Types (Parser)
+import Data.Aeson.Types (Parser, formatError, iparse)
 import Data.Aeson.Parser (eitherDecodeWith)
-import Data.Aeson.Internal (formatError, iparse)
 import Data.Algorithm.Diff (PolyDiff (..), getGroupedDiff)
 import Data.Proxy (Proxy(..))
 import Data.Semigroup ((<>))
@@ -31,6 +30,7 @@ tests :: [TestTree]
 tests =
   [ aesonGoldenTest "simple" "tests/golden/simple.expected" output
   , aesonGoldenTest "generic" "tests/golden/generic.expected" (outputGeneric G)
+  , aesonGoldenTest "generic" "tests/golden/th.expected" (outputGeneric TH)
   ]
 
 output :: Output
@@ -140,7 +140,7 @@ outputGeneric choice = concat
       (select
         thSomeTypeParseJSONRejectUnknownFields
         gSomeTypeParseJSONRejectUnknownFields)
-      [ "{\"tag\": \"record\", \"testOne\": 1.0, \"testZero\": 1}"
+      [ "{\"tag\": \"record\", \"testone\": 1.0, \"testZero\": 1}"
       , "{\"testZero\": 1}"
       , "{\"tag\": \"record\", \"testone\": true, \"testtwo\": null, \"testthree\": null}"
       ]
