@@ -11,16 +11,18 @@ import Test.QuickCheck (Property)
 import PropUtils
 import PropertyGeneric
 import PropertyKeys
+import PropertyQC
 import PropertyRoundTrip
 import PropertyTH
 
 
 tests :: TestTree
 tests = testGroup "properties" [
-  testGroup "encode" [
-      testProperty "encodeDouble" encodeDouble
+  testGroup "encode"
+    [ testProperty "encodeDouble" encodeDouble
     , testProperty "encodeInteger" encodeInteger
     ]
+  , testProperty "read . show = id" roundtripReadShow
   , roundTripTests
   , keysTests
   , testGroup "toFromJSON" [
@@ -37,4 +39,5 @@ tests = testGroup "properties" [
     ]
   , genericTests
   , templateHaskellTests
+  , quickcheckTests
   ]
